﻿using System;
using Infragistics.Silverlight;
using System.Collections.Generic;
using na2010v1.NorthwindService;
using na2010v1.Model;

namespace na2010v1.Views.xamWebGrid.VirtualCollectionPaging
{
    public class VirtualCollectionPagingViewModel : ViewModelBase
    {
        private VirtualCollection<ProductDTO> _items = null;
        private NorthwindClient _client = null;
        private int _resultTotalCount = 0;
        private bool _isLoading = true;
        private string _busyText = "Loading...";

        public VirtualCollection<ProductDTO> Items
        {
            get { return this._items; }
        }

        public bool IsLoading
        {
            get { return this._isLoading; }
            private set
            {
                if (this._isLoading != value)
                {
                    this._isLoading = value;
                    this.RaisePropertyChangedEvent("IsLoading");
                }
            }
        }

        public string BusyText
        {
            get { return this._busyText; }
            private set
            {
                if (this._busyText != value)
                {
                    this._busyText = value;
                    this.RaisePropertyChangedEvent("BusyText");
                }
            }
        }

        public VirtualCollectionPagingViewModel(int pageSize)
        {
            this._items = new VirtualCollection<ProductDTO>(100, pageSize);
            this._items.PageSize = pageSize;
            this._items.ItemDataRequested += new EventHandler<ItemDataRequestedEventArgs>(_items_ItemDataRequested);

            this._client = new NorthwindClient();
            this._client.GetProductsForPagingCompleted += 
                new EventHandler<GetProductsForPagingCompletedEventArgs>(_client_GetProductsForPagingCompleted);
        }

        private void _items_ItemDataRequested(object sender, ItemDataRequestedEventArgs e)
        {
            this.NotifyLoadStart("Loading Products...");
            this._client.GetProductsForPagingAsync(this._items.PageSize, e.StartIndex, this._resultTotalCount);
        }

        private void _client_GetProductsForPagingCompleted(object sender, GetProductsForPagingCompletedEventArgs e)
        {
            this._resultTotalCount = e.Result.TotalItemsCount;
            this._items.AnticipatedCount = this._resultTotalCount;
            this._items.LoadItems(e.Result.StartIndex, e.Result.DataItems);
            this.NotifyLoadStop();
        }

        private void NotifyLoadStart(string message)
        {
            this.IsLoading = true;
            this.BusyText = message;
        }

        private void NotifyLoadStop()
        {
            this.IsLoading = false;
        }
    }
}